<?php

namespace Drupal\restrict_route_by_ip\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Routing\RouteBuilderInterface;
use Drupal\restrict_route_by_ip\Entity\RestrictRouteInterface;
use Drupal\restrict_route_by_ip\Service\RestrictIpInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Set status on RestrictRoute.
 */
class StatusController extends ControllerBase {

  /**
   * The Drupal RouteBuilder service.
   *
   * @var \Drupal\Core\Routing\RouteBuilderInterface
   */
  protected RouteBuilderInterface $routeBuilder;

  /**
   * The Drupal RequestStack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected RequestStack $requestStack;

  /**
   * The RestrictRouteByIp service.
   *
   * @var \Drupal\restrict_route_by_ip\Service\RestrictIpInterface
   */
  protected RestrictIpInterface $restrictIpService;

  /**
   * StatusController constructor.
   *
   * @param \Drupal\Core\Routing\RouteBuilderInterface $route_builder
   *   The form builder.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The Drupal RequestStack service.
   * @param \Drupal\restrict_route_by_ip\Service\RestrictIpInterface $restrict_ip_service
   *   The RestrictRouteByIp service.
   */
  public function __construct(
    RouteBuilderInterface $route_builder,
    RequestStack $request_stack,
    RestrictIpInterface $restrict_ip_service,
  ) {
    $this->routeBuilder = $route_builder;
    $this->requestStack = $request_stack;
    $this->restrictIpService = $restrict_ip_service;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The Drupal service container.
   *
   * @return static
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('router.builder'),
      $container->get('request_stack'),
      $container->get('restrict_route_by_ip.service')
    );
  }

  /**
   * Enables a restricted route by setting its status to TRUE.
   *
   * Saves the changes, displays a confirmation message, rebuilds the route
   * collection, and redirects to the restricted route collection page.
   *
   * @param \Drupal\restrict_route_by_ip\Entity\RestrictRouteInterface $restrict_route
   *   The restricted route entity to enable.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response to the restricted route collection page.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   *   Thrown if there is an issue saving the entity.
   */
  public function enable(RestrictRouteInterface $restrict_route): RedirectResponse {
    $restrict_route->setStatus(TRUE);
    $restrict_route->save();
    $this->messenger()->addMessage($this->t('Route @label enabled.', [
      '@label' => $restrict_route->label(),
    ]));
    $this->routeBuilder->rebuild();
    return $this->redirect('entity.restrict_route.collection');
  }

  /**
   * Disables a restricted route by setting its status to FALSE.
   *
   * Displays a confirmation message to the user, rebuilds the route collection,
   * and redirects the user to the restricted route collection page.
   *
   * @param \Drupal\restrict_route_by_ip\Entity\RestrictRouteInterface $restrict_route
   *   The restricted route entity to disable.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response to the restricted route collection page.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function disable(RestrictRouteInterface $restrict_route): RedirectResponse {
    $restrict_route->setStatus(FALSE);
    $restrict_route->save();
    $this->messenger()->addMessage($this->t('Route @label disabled.', [
      '@label' => $restrict_route->label(),
    ]));
    $this->routeBuilder->rebuild();
    return $this->redirect('entity.restrict_route.collection');
  }

  /**
   * Handles the impacted paths based on the current request.
   *
   * Returns a JSON response indicating whether the operation was successful
   * and, if successful, the paths impacted by the provided route name.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function impactedPath(): JsonResponse {
    $current_request = $this->requestStack->getCurrentRequest();
    $response = new JsonResponse([
      'status' => 'ko',
    ]);
    if ($current_request->isXmlHttpRequest()) {
      $route_name = $current_request->request->get('route_name');
      $restricted_routes = $this->restrictIpService->getAllRouteNamesAndPath($route_name);
      $response = new JsonResponse([
        'status' => 'ok',
        'impacted' => $restricted_routes['path'],
      ]);
    }
    return $response;
  }

}
