<?php

namespace Drupal\Tests\restrict_route_by_ip\Unit;

use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\restrict_route_by_ip\Entity\RestrictRouteInterface;
use Drupal\restrict_route_by_ip\Form\RestrictRouteGlobalForm;
use Drupal\restrict_route_by_ip\Service\RestrictIpInterface;
use Drupal\restrict_route_by_ip\Service\RestrictIpService;
use Drupal\Tests\UnitTestCase;
use Prophecy\PhpUnit\ProphecyTrait;
use Psr\Log\LoggerInterface;

/**
 * Tests of the RestrictIpService class.
 *
 * @coversDefaultClass \Drupal\restrict_route_by_ip\Service\RestrictIpService
 * @group restrict_route_by_ip
 */
class RestrictIpTest extends UnitTestCase {

  /**
   * Required modules for the test.
   *
   * @var array
   */
  protected static array $modules = ['restrict_route_by_ip'];

  /**
   * The Restrict IP Service instance being tested.
   *
   * @var \Drupal\restrict_route_by_ip\Service\RestrictIpInterface
   */
  protected RestrictIpInterface $restrictIpService;

  /**
   * Mocked configuration object used in tests.
   *
   * @var \Prophecy\Prophecy\ObjectProphecy|\Drupal\Core\Config\Config
   */
  protected $configObject;

  /**
   * Mocked entity storage service used to manage entities in tests.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $entityStorage;

  /**
   * Mock route match interface to simulate current route matching in tests.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $routeMatch;

  /**
   * The HTTP request object used in the current context.
   *
   * @var \Symfony\Component\HttpFoundation\Request|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $request;
  /**
   * Restrict route entity mock (first route).
   *
   * @var \Drupal\restrict_route_by_ip\Entity\RestrictRouteInterface
   */
  protected RestrictRouteInterface $restrictRouteEntity1;

  /**
   * Restrict route entity mock (second route).
   *
   * @var \Drupal\restrict_route_by_ip\Entity\RestrictRouteInterface
   */
  protected RestrictRouteInterface $restrictRouteEntity2;

  /**
   * Initial data for global configuration.
   *
   * @var array
   */
  protected array $globalConfigInitialData = [
    RestrictRouteGlobalForm::FIELD_STATUS => RestrictRouteGlobalForm::STATUS_ENABLE,
    RestrictRouteGlobalForm::FIELD_DEBUG_MODE => FALSE,
  ];

  /**
   * Sets up the environment before each test.
   */
  public function setUp(): void {
    parent::setUp();

    // Mock the config object.
    $this->configObject = $this->prophesize(Config::class);
    $config_factory = $this->prophesize(ConfigFactoryInterface::class);
    $config_factory->get(RestrictRouteGlobalForm::CONFIG_NAME)
      ->willReturn($this->configObject->reveal());

    // Mock the request stack.
    $this->request = $this->prophesize('\Symfony\Component\HttpFoundation\Request');
    $request_stack = $this->prophesize('\Symfony\Component\HttpFoundation\RequestStack');
    $request_stack->getCurrentRequest()->willReturn($this->request->reveal());

    // Mock the logger factory.
    $logger = $this->prophesize(LoggerInterface::class);
    $logger_factory = $this->prophesize(LoggerChannelFactoryInterface::class);
    $logger_factory->get('restrict_route_by_ip')->willReturn($logger->reveal());

    // Mock the route match interface.
    $this->routeMatch = $this->prophesize(RouteMatchInterface::class);

    // Mock the entity type manager and restrict route entities.
    $restrict_route_entity = $this->prophesize(RestrictRouteInterface::class);
    $restrict_route_entity->getRoute()->willReturn('admin.test');
    $restrict_route_entity->id()->willReturn('1');
    $restrict_route_entity->getIps()->willReturn([
      '123.123.123.1',
      '123.123.123.2',
    ]);
    $this->restrictRouteEntity1 = $restrict_route_entity->reveal();

    $restrict_route_entity2 = $this->prophesize(RestrictRouteInterface::class);
    $restrict_route_entity2->getRoute()->willReturn('admin.test2');
    $restrict_route_entity2->id()->willReturn('2');
    $restrict_route_entity2->getIps()->willReturn(['123.123.123.123']);
    $this->restrictRouteEntity2 = $restrict_route_entity2->reveal();

    // Mock the entity storage.
    $this->entityStorage = $this->prophesize(EntityStorageInterface::class);

    $entity_type_manager = $this->prophesize(EntityTypeManagerInterface::class);
    $entity_type_manager->getStorage('restrict_route')
      ->willReturn($this->entityStorage->reveal());

    // Mock the route provider and routes.
    $route1 = $this->prophesize('\Symfony\Component\Routing\Route');
    $route1->getPath()->willReturn('/admin/test');

    $route2 = $this->prophesize('\Symfony\Component\Routing\Route');
    $route2->getPath()->willReturn('/admin/test/2');

    $route3 = $this->prophesize('\Symfony\Component\Routing\Route');
    $route3->getPath()->willReturn('/my/test/3');

    $route4 = $this->prophesize('\Symfony\Component\Routing\Route');
    $route4->getPath()->willReturn('/my/param/{number}');

    $route_provider = $this->prophesize('\Drupal\Core\Routing\RouteProvider');
    $route_provider->getAllRoutes()->willReturn([
      'admin.test' => $route1->reveal(),
      'admin.test2' => $route2->reveal(),
      'admin.test3' => $route3->reveal(),
      'route.param' => $route3->reveal(),
    ]);

    // Mock the messenger service.
    $messenger = $this->prophesize(MessengerInterface::class);

    // Instantiate the service being tested.
    $this->restrictIpService = new RestrictIpService(
      $config_factory->reveal(),
      $request_stack->reveal(),
      $logger_factory->reveal(),
      $this->routeMatch->reveal(),
      $entity_type_manager->reveal(),
      $route_provider->reveal(),
      $messenger->reveal());
  }

  /**
   * Tests the userIpIsRestricted method.
   *
   * @covers \Drupal\restrict_route_by_ip\Service\RestrictIpService::userIpIsRestricted
   */
  public function testUserIpIsRestricted(): void {
    // Mock the account object (not used in this test).
    $account = $this->prophesize(AccountInterface::class);

    // Mock the QueryInterface and storage.
    $this->entityQueryExpects();

    // Test IP restriction for the first route.
    $this->routeMatch->getRouteName()->willReturn('admin.test');
    $this->request->getClientIp()->willReturn('123.123.123.123');
    $this->entityStorage->load('1')->willReturn($this->restrictRouteEntity1);
    $this->entityStorage->load('2')->willReturn($this->restrictRouteEntity2);

    $ip_is_restricted = $this->restrictIpService->userIpIsRestricted($account->reveal());
    $this->assertTrue($ip_is_restricted, 'User ip in restricted ips.');

    // Test IP that is not restricted.
    $this->request->getClientIp()->willReturn('123.123.123.1');
    $ip_is_not_restricted = $this->restrictIpService->userIpIsRestricted($account->reveal());
    $this->assertFalse($ip_is_not_restricted, 'User ip not in restricted ips.');
  }

  /**
   * Tests the getAllRouteNamesAndPath method.
   *
   * @covers \Drupal\restrict_route_by_ip\Service\RestrictIpService::getAllRouteNamesAndPath
   */
  public function testGetAllRouteNamesAndPath(): void {
    // Test retrieving route names and paths for a specific route.
    $name_and_path = $this->restrictIpService->getAllRouteNamesAndPath('admin.test2');

    // Check that the result array has 'name' and 'path' as keys.
    $this->assertArrayHasKey('name', $name_and_path, 'Route detail as key name.');
    $this->assertArrayHasKey('path', $name_and_path, 'Route names and paths has key path.');
    // Verify the route name and path.
    $this->assertEquals(['admin.test2'], $name_and_path['name'], 'A route name should return the same route in array.');
    $this->assertEquals(['/admin/test/2'], $name_and_path['path'], 'A route name should return the only one route path in array.');

    // Test for a non-existing route.
    $name_and_path = $this->restrictIpService->getAllRouteNamesAndPath('admin.test.dummy');
    // Ensure non-existing routes return empty arrays for 'name' and 'path'.
    $this->assertEquals([], $name_and_path['name'], 'A non existing route should return an empty array for key "name".');
    $this->assertEquals([], $name_and_path['path'], 'A non existing route should return an empty array for key "path".');

    // Test using a path with a wildcard.
    $name_and_path = $this->restrictIpService->getAllRouteNamesAndPath('/admin/%');
    // Check that wildcard paths return all matching route names and paths.
    $this->assertEquals([
      'admin.test',
      'admin.test2',
    ], $name_and_path['name'], 'A path with wildcard return searched routes names.');
    $this->assertEquals([
      '/admin/test',
      '/admin/test/2',
    ], $name_and_path['path'], 'A path with wildcard return searched routes path.');

    // Test using a path without a wildcard.
    $name_and_path = $this->restrictIpService->getAllRouteNamesAndPath('/admin/');
    // Ensure it returns the same as the wildcard ending.
    $this->assertEquals([
      'admin.test',
      'admin.test2',
    ], $name_and_path['name'], 'A path without wildcard return routes names same than wildcard at the end.');
    $this->assertEquals([
      '/admin/test',
      '/admin/test/2',
    ], $name_and_path['path'], 'A path without wildcard return routes path same than wildcard at the end.');

    // Test using a regex pattern in the path.
    $name_and_path = $this->restrictIpService->getAllRouteNamesAndPath('/[a-z]{3,}/test');
    // Verify that the regex returns the correct route names and paths.
    $this->assertEquals([
      'admin.test',
      'admin.test2',
    ], $name_and_path['name'], 'Check route names return for regex.');
    $this->assertEquals([
      '/admin/test',
      '/admin/test/2',
    ], $name_and_path['path'], 'Check route paths return for regex.');
  }

  /**
   * Tests the getRestrictedRouteDetail method.
   *
   * @covers \Drupal\restrict_route_by_ip\Service\RestrictIpService::getRestrictedRouteDetail
   */
  public function testGetRestrictedRouteDetail(): void {
    // Create mock of the RestrictRouteInterface.
    $restrict_route_entity = $this->prophesize(RestrictRouteInterface::class);
    $restrict_route_entity->getRoute()->willReturn('admin.test');
    $restrict_route_entity->id()->willReturn('1');

    // Retrieve restricted route details using the mock.
    $detail = $this->restrictIpService->getRestrictedRouteDetail($restrict_route_entity->reveal());
    // Check that the returned array contains specific keys.
    $this->assertArrayHasKey('restricted_route_id', $detail, 'Route detail has key restricted_route_id.');
    $this->assertArrayHasKey('route_names', $detail, 'Route detail has key route_names.');
    $this->assertArrayHasKey('path', $detail, 'Route detail has key path.');

    // Verify the restricted route ID, names, and paths.
    $this->assertEquals('1', $detail['restricted_route_id'], 'Get restricted route id.');
    $this->assertEquals(['admin.test'], $detail['route_names'], 'Get restricted route names.');
    $this->assertEquals(['/admin/test'], $detail['path'], 'Get restricted route paths.');
  }

  /**
   * Tests the getRestrictedRouteId method.
   *
   * @covers \Drupal\restrict_route_by_ip\Service\RestrictIpService::getRestrictedRouteId
   */
  public function testGetRestrictedRouteId(): void {
    // Mock the QueryInterface and storage.
    $this->entityQueryExpects();

    // Test retrieving the restricted route ID for an existing route.
    $id = $this->restrictIpService->getRestrictedRouteId('admin.test');
    $this->assertEquals('1', $id, 'Get restricted route id.');

    // Test for a non-existing route ID.
    $id = $this->restrictIpService->getRestrictedRouteId('dummy.test');
    $this->assertEquals(NULL, $id, 'Get no restricted route for dummy id.');
  }

  /**
   * Tests the getAllRestrictedRoutes method.
   *
   * @covers \Drupal\restrict_route_by_ip\Service\RestrictIpService::getAllRestrictedRoutes
   */
  public function testGetAllRestrictedRoutes(): void {
    // Mock the QueryInterface and storage.
    $this->entityQueryExpects();

    // Simulate loading the entities and verify the count.
    $this->entityStorage->loadMultiple(['1', '2'])
      ->willReturn([$this->restrictRouteEntity1, $this->restrictRouteEntity2]);

    $routes = $this->restrictIpService->getAllRestrictedRoutes();
    $this->assertEquals(2, count($routes), 'Get all available restricted routes.');

    // Test when the global restriction is disabled.
    $this->configObject->get(RestrictRouteGlobalForm::FIELD_STATUS)
      ->willReturn(RestrictRouteGlobalForm::STATUS_DISABLE);

    $no_route = $this->restrictIpService->getAllRestrictedRoutes();
    $this->assertEquals(0, count($no_route), 'Should no return any route due to disable configuration.');
  }

  /**
   * Tests the checkRangeIp method with enabled configurations.
   *
   * @covers \Drupal\restrict_route_by_ip\Service\RestrictIpService::checkRangeIp
   */
  public function testEnableCheckRangeIp(): void {
    // Mock configuration to enable IP checking.
    $this->configObject->get(RestrictRouteGlobalForm::FIELD_STATUS)
      ->willReturn(RestrictRouteGlobalForm::STATUS_ENABLE);

    // Test with a simple IP match.
    $simple_ip = $this->restrictIpService->checkRangeIp('84.34.35.23', '84.34.35.23');
    $this->assertTrue($simple_ip, 'Check simple ip address.');

    // Test with a non-matching IP.
    $simple_false_ip = $this->restrictIpService->checkRangeIp('84.34.35.23', '84.34.35.22');
    $this->assertFalse($simple_false_ip, 'Check simple wrong ip address.');

    // Test with an IP inside a CIDR range.
    $inside_24 = $this->restrictIpService->checkRangeIp('84.34.35.2', '84.34.35.1/24');
    $this->assertTrue($inside_24, 'Check ip address is inside a range (CIDR format).');

    // Test with an IP outside a CIDR range.
    $outside_24 = $this->restrictIpService->checkRangeIp('84.34.36.2', '84.34.35.1/24');
    $this->assertFalse($outside_24, 'Check ip address is outside a range (CIDR format).');

    // Test with an IP inside a start-end range.
    $inside_wildcard = $this->restrictIpService->checkRangeIp('84.34.35.22', '84.34.35.1-84.34.35.23');
    $this->assertTrue($inside_wildcard, 'Check ip address is inside a range (start-end format).');

    // Test with an IP outside a start-end range.
    $inside_wildcard = $this->restrictIpService->checkRangeIp('84.34.35.24', '84.34.35.1-84.34.35.23');
    $this->assertFalse($inside_wildcard, 'Check if the IP is outside a start-end range.');

    // Test with an IP inside a wildcard range.
    $inside_wildcard = $this->restrictIpService->checkRangeIp('84.34.35.2', '84.34.35.*');
    $this->assertTrue($inside_wildcard, 'Check if the IP is inside a wildcard range.');

    // Test with an IP outside a wildcard range.
    $outside_wildcard = $this->restrictIpService->checkRangeIp('84.34.36.5', '84.34.35.*');
    $this->assertFalse($outside_wildcard, 'Check if the IP is outside a wildcard range.');
  }

  /**
   * Tests IP range checking when the IP restriction is disabled on localhost.
   *
   * @covers \Drupal\restrict_route_by_ip\Service\RestrictIpService::checkRangeIp
   */
  public function testDisableOnLocalhostCheckRangeIp(): void {
    // Mock configuration to disable IP checking on localhost.
    $this->configObject->get(RestrictRouteGlobalForm::FIELD_STATUS)
      ->willReturn(RestrictRouteGlobalForm::STATUS_DISABLE_ON_LOCALHOST);

    // Test with a simple IP match.
    $simple_ip = $this->restrictIpService->checkRangeIp('84.34.35.23', '84.34.35.23');
    $this->assertTrue($simple_ip, 'Check if the exact IP matches.');

    // Test with an IP inside a CIDR range.
    $inside_24 = $this->restrictIpService->checkRangeIp('84.34.35.2', '84.34.35.1/24');
    $this->assertTrue($inside_24, 'Check if the IP is inside a CIDR range.');

    // Test with an IP inside a start-end range.
    $inside_wildcard = $this->restrictIpService->checkRangeIp('84.34.35.22', '84.34.35.1-84.34.35.23');
    $this->assertTrue($inside_wildcard, 'Check if the IP is inside a start-end range.');

    // Test with an IP inside a wildcard range.
    $inside_wildcard = $this->restrictIpService->checkRangeIp('84.34.35.2', '84.34.35.*');
    $this->assertTrue($inside_wildcard, 'Check if the IP is inside a wildcard range.');

    // Test with a simple IP match on localhost (127.0.0.1).
    $simple_ip = $this->restrictIpService->checkRangeIp('127.0.0.1', '84.34.35.23');
    $this->assertTrue($simple_ip, 'Check if localhost IP (127.0.0.1) is allowed.');

    // Test with a localhost IP inside a CIDR range.
    $inside_24 = $this->restrictIpService->checkRangeIp('127.0.0.1', '84.34.35.1/24');
    $this->assertTrue($inside_24, 'Check if localhost IP (127.0.0.1) is allowed inside a CIDR range.');

    // Test with a localhost IP inside a start-end range.
    $inside_wildcard = $this->restrictIpService->checkRangeIp('127.0.0.1', '84.34.35.1-84.34.35.23');
    $this->assertTrue($inside_wildcard, 'Check if localhost IP (127.0.0.1) is allowed inside a start-end range.');

    // Test with a localhost IP inside a wildcard range.
    $inside_wildcard = $this->restrictIpService->checkRangeIp('127.0.0.1', '84.34.35.*');
    $this->assertTrue($inside_wildcard, 'Check if localhost IP (127.0.0.1) is allowed inside a wildcard range.');

    // Test with a simple IP match on IPv6 localhost (::1).
    $simple_ip = $this->restrictIpService->checkRangeIp('::1', '84.34.35.23');
    $this->assertTrue($simple_ip, 'Check if IPv6 localhost IP (::1) is allowed.');

    // Test with an IPv6 localhost IP inside a CIDR range.
    $inside_24 = $this->restrictIpService->checkRangeIp('::1', '84.34.35.1/24');
    $this->assertTrue($inside_24, 'Check if IPv6 localhost IP (::1) is allowed inside a CIDR range.');

    // Test with an IPv6 localhost IP inside a start-end range.
    $inside_wildcard = $this->restrictIpService->checkRangeIp('::1', '84.34.35.1-84.34.35.23');
    $this->assertTrue($inside_wildcard, 'Check if IPv6 localhost IP (::1) is allowed inside a start-end range.');

    // Test with an IPv6 localhost IP inside a wildcard range.
    $inside_wildcard = $this->restrictIpService->checkRangeIp('::1', '84.34.35.*');
    $this->assertTrue($inside_wildcard, 'Check if IPv6 localhost IP (::1) is allowed inside a wildcard range.');
  }

  /**
   * Sets up expectations for how the `entityStorage` service behaves in tests.
   */
  protected function entityQueryExpects(): void {
    // Mocked the queryInterface for retrieving routes.
    $query_interface = $this->prophesize(QueryInterface::class);
    $query_interface->condition('status', '1')
      ->willReturn($query_interface->reveal());
    $query_interface->accessCheck()->willReturn($query_interface);
    $query_interface->execute()->willReturn(['1', '2']);
    $this->entityStorage->getQuery()->willReturn($query_interface->reveal());

    // Simulate loading the entities and verify the count.
    $this->entityStorage->loadMultiple(['1', '2'])
      ->willReturn([$this->restrictRouteEntity1, $this->restrictRouteEntity2]);
  }

}
